<template>
    <div class="page-container">
        <div class="chart-container">
            <div style="width: 60%; display: flex">
                <div class="chard">
                    <div class="main-card2">
                        <div class="chart-title">优化前</div>
                        <div class="content-card">
                            <div id="container"></div>
                        </div>
                    </div>
                </div>
                <div class="chard1">
                    <div class="main-card2">
                        <div class="chart-title">优化后</div>
                        <div class="content-card">
                            <div id="container1"></div>
                        </div>
                    </div>
                </div>
            </div>

            <div class="chard2">
                <div class="main-card2">
                    <div class="content-title">
                        <div class="title-left">
                            <span>30天节省</span>
                            <span>过去30天节省趋势</span>
                        </div>
                        <div style="display: flex">
                            <div class="title-right">
                                <div class="title-right-span">
                                    <span>CO2排放</span>
                                    <span>单位：吨</span>
                                </div>
                                <div class="title-right-div">{{ co2 }}</div>
                            </div>
                            <div class="title-right1">
                                <div class="title-right-span">
                                    <span>运维成本</span>
                                    <span>单位：万元</span>
                                </div>
                                <div class="title-right-div">{{ cost }}</div>
                            </div>
                        </div>
                    </div>
                    <div class="content-card1">
                        <div id="container2"></div>
                    </div>
                </div>
            </div>
        </div>
    </div>
</template>

<script>
import {Chart} from '@antv/g2'
export default {
    name: 'chart-dialog',
    data() {
        return {
            co2: 0,
            cost: 0,
            befortotalcost: 0, // 优化前费用
            aftertotalcost: 0, // 优化后总费用
            statisticsRate: {}, //环图统计列表
            statisticsTotal: [], //柱状图统计列表
            statisticsData: {},
            beforeData: [
                {item: '交通成本', count: 0, percent: 0},
                {item: '人工成本', count: 0, percent: 0},
                {item: '发电损失', count: 0, percent: 0},
            ],
            afterData: [
                {item: '交通成本', count: 0, percent: 0},
                {item: '人工成本', count: 0, percent: 0},
                {item: '发电损失', count: 0, percent: 0},
            ],
            chartBefore: null,
            chartAfter: null,
            dataOrigin: {
                aftertotalcost: 0,
                afterhumancostrate: 0,
                afterroutecostrate: 0,
                afterwindpowerrate: 0,
                befortotalcost: 0,
                beforhumancostrate: 0,
                beforroutecostrate: 0,
                beforwindpowerrate: 0,
            },
            barChart: null,
            barData: [{date: '', costvalue: 0}],
            optimization: 0
        }
    },
    mounted() {
        this.$nextTick(() => {
            this.initChart()
            this.initChart1()
            this.initChart2()
        })
    },
    methods: {
        changeData(data) {
            this.formatStatistics(data.statisticsRate)
            if (data.statisticsTotal) {
                this.barData = data.statisticsTotal
                this.calculateData(data.statisticsTotal)
            } else {
                this.barData = [{date: '', costvalue: 0}]
            }
            this.barChart.changeData(this.barData)
        },
        calculateData(data) {
            this.co2 = 0
            this.cost = 0
            let beforewindpower = 0
            let windpower = 0
            let beforeroutecost = 0
            let routecost = 0
            data.forEach((item) => {
                this.cost += item.costvalue
                beforewindpower += item.beforewindpower
                windpower += item.windpower
                beforeroutecost += item.beforeroutecost
                routecost += item.routecost
            })
            this.co2 = (
                ((beforewindpower - windpower) / 0.85 +
                    ((beforeroutecost - routecost) * 2.63) / 7) /
                10000
            ).toFixed(2)
            this.cost = (this.cost / 10000).toFixed(2)
            console.log(
                this.co2,
                this.cost,
                beforewindpower,
                windpower,
                beforeroutecost,
                routecost
            )
        },
        formatStatistics(data) {
            if (data && data.befortotalcost && data.aftertotalcost) {
                for (let key in data) {
                    data[key] = parseFloat(data[key])
                }

                this.befortotalcost = data.befortotalcost
                this.aftertotalcost = data.aftertotalcost

                this.beforeData[0].count = data.beforroutecostrate
                this.beforeData[0].percent = (
                    data.beforroutecostrate / data.befortotalcost
                ).toFixed(4)

                this.beforeData[1].count = data.beforhumancostrate
                this.beforeData[1].percent = (
                    data.beforhumancostrate / data.befortotalcost
                ).toFixed(4)

                this.beforeData[2].count = data.beforwindpowerrate
                this.beforeData[2].percent = (
                    data.beforwindpowerrate / data.befortotalcost
                ).toFixed(4)

                this.afterData[0].count = data.afterroutecostrate
                this.afterData[0].percent = (
                    data.afterroutecostrate / data.aftertotalcost
                ).toFixed(4)

                this.afterData[1].count = data.afterhumancostrate
                this.afterData[1].percent = (
                    data.afterhumancostrate / data.aftertotalcost
                ).toFixed(4)

                this.afterData[2].count = data.afterwindpowerrate
                this.afterData[2].percent = (
                    data.afterwindpowerrate / data.aftertotalcost
                ).toFixed(4);

                this.optimization = ((1 - this.aftertotalcost / this.befortotalcost) *  100).toFixed(2)
            } else {
                this.befortotalcost = 0
                this.aftertotalcost = 0
                this.optimization = 0
                this.beforeData = [
                    {item: '交通成本', count: 0, percent: 0},
                    {item: '人工成本', count: 0, percent: 0},
                    {item: '发电损失', count: 0, percent: 0},
                ]
                this.afterData = [
                    {item: '交通成本', count: 0, percent: 0},
                    {item: '人工成本', count: 0, percent: 0},
                    {item: '发电损失', count: 0, percent: 0},
                ]
            }

            console.log(this.beforeData, 'this.beforeData', this.afterData)
            this.chartBefore.changeData(this.beforeData)
            this.chartAfter.changeData(this.afterData)
        },
        initChart() {
            this.chartBefore = new Chart({
                container: 'container',
                autoFit: true,
                height: 500,
            })

            this.chartBefore.coordinate('theta', {
                radius: 0.6,
                innerRadius: 0.6,
            })
            this.chartBefore.data(this.beforeData)
            this.chartBefore.scale('count', {
                formatter: (val) => {
                    return val
                },
            })
            this.chartBefore.legend('item', {
                itemName: {
                    style: {
                        fill: '#ccf2ff',
                    },
                },
                itemValue: {
                    style: {
                        fill: '#23bbff',
                    }, // 配置 itemValue 样式
                    formatter: (text, item, index) => {
                        let textShow =
                            (
                                parseFloat(this.beforeData[index].percent) * 100
                            ).toFixed(2) + ''
                        return textShow.slice(0, 5) + '%'
                    },
                },
                position: 'right', // 配置图例显示位置
                offsetX: -45,
                offsetY: 15,
            })
            this.chartBefore
                .annotation()
                .text({
                    position: ['70%', '40%'],
                    content: '总费用：',
                    style: {
                        fontSize: 14,
                        fill: '#ccf2ff',
                    },
                    offsetX: 47,
                    offsetY: -25,
                })
                .text({
                    position: ['85%', '40%'],
                    content: (obj) => {
                        return this.befortotalcost + '元'
                    },
                    style: {
                        fontSize: 14,
                        fill: '#23bbff',
                    },
                    offsetX: 50,
                    offsetY: -25,
                })
            this.chartBefore.tooltip({
                showTitle: false,
                showMarkers: false,
                domStyles: {
                    'g2-tooltip': {
                        background: 'rgba(0,0,0, 0.8)',
                        color: '#d6e0ec',
                    },
                },
            })
            this.chartBefore
                .interval()
                .position('count')
                .color('item')
                .adjust('stack')
            this.chartBefore.interaction('element-active')
            this.chartBefore.render()
        },
        initChart1() {
            this.chartAfter = new Chart({
                container: 'container1',
                autoFit: true,
                height: 500,
            })
            this.chartAfter.coordinate('theta', {
                radius: 0.6,
                innerRadius: 0.6,
            })
            this.chartAfter.data(this.afterData)
            this.chartAfter.scale('count', {
                formatter: (val) => {
                    return val
                },
            })
            this.chartAfter.legend('item', {
                itemName: {
                    style: {
                        fill: '#ccf2ff',
                    },
                },
                itemValue: {
                    style: {
                        fill: '#23bbff',
                    }, // 配置 itemValue 样式
                    formatter: (text, item, index) => {
                        let showText =
                            (
                                parseFloat(this.afterData[index].percent) * 100
                            ).toFixed(2) + ''
                        return showText.slice(0, 5) + '%'
                    }, // 格式化 itemValue 内容
                },
                position: 'right', // 配置图例显示位置
                offsetX: -45, //图例与图表的距离
                offsetY: 15,
            })
            this.chartAfter.tooltip({
                showTitle: false,
                showMarkers: false,
                domStyles: {
                    'g2-tooltip': {
                        background: 'rgba(0,0,0, 0.8)',
                        color: '#d6e0ec',
                    },
                },
            })
            this.chartAfter
                .annotation()
                .text({
                    position: ['70%', '35%'],
                    content: '总费用：',
                    style: {
                        fontSize: 14,
                        fill: '#ccf2ff',
                    },
                    offsetX: 47,
                    offsetY: -25,
                })
                .text({
                    position: ['85%', '35%'],
                    content: () => this.aftertotalcost + '元',
                    style: {
                        fontSize: 14,
                        fill: '#23bbff',
                    },
                    offsetX: 50,
                    offsetY: -25,
                })
                .text({
                    position: ['70%', '35%'],
                    content: '优化率：',
                    style: {
                        fontSize: 14,
                        fill: '#ccf2ff',
                    },
                    offsetX: 47,
                    offsetY: -2,
                })
                .text({
                    position: ['85%', '35%'],
                    content: () => this.optimization + '%',
                    style: {
                        fontSize: 14,
                        fill: '#23bbff',
                    },
                    offsetX: 50,
                    offsetY: -2,
                })
            this.chartAfter
                .interval()
                .position('count')
                .color('item')
                .adjust('stack')
            this.chartAfter.interaction('element-active')
            this.chartAfter.render()
        },
        initChart2() {
            this.barChart = new Chart({
                container: 'container2',
                autoFit: true,
                height: 500,
            })
            this.barChart.data(this.barData)

            this.barChart.scale('costvalue', {
                tickLine: null,
                alias: '成本',
                nice: true,
            })
            this.barChart.axis('date', {
                tickLine: null,
            })

            this.barChart.axis('costvalue', {
                label: {
                    style: {
                        fill: '#979ba4',
                    },
                },
                line: {
                    style: {
                        stroke: '#414959',
                    },
                },
                grid: {
                    line: {
                        style: {
                            stroke: '#414959',
                            lineWidth: 1,
                            lineDash: [3, 3],
                        },
                    },
                    alignTick: true,
                },
            })

            this.barChart.legend(false)
            this.barChart.tooltip({
                shared: false,
                showMarkers: false,
                domStyles: {
                    'g2-tooltip': {
                        background: 'rgba(0,0,0, 0.8)',
                        color: '#d6e0ec',
                    },
                },
            })

            this.barChart.interaction('active-region')

            this.barChart.interval().position('date*costvalue')

            this.barChart.render()
        },
    },
}
</script>

<style lang="scss" scoped>
@import '@/assets/css/element-variables.scss';
.page-container {
  height: 100%;
  width: 100%;
}
.chart-container {
  width: 100%;
  height: 100%;
  display: flex;
}
#container,
#container1,
#container2 {
  width: 100%;
  height: 100%;
}
.chart-title {
  padding-left: 100px;
  padding-top: 20px;
  line-height: 20px;
  color: #ccf2ff;
  font-size: 16px;
  font-weight: bold;
}

.chard2 {
  width: 40%;
  height: 100%;
  padding-left: 20px;
}
.chard,
.chard1 {
  width: 50%;
  height: 100%;
}
.main-card2 {
  height: calc(100% - 40px);
}
.content-title {
  padding-top: 20px;
  line-height: 20px;
  color: $--color-text;
  display: flex;
  justify-content: space-between;
  align-items: center;
}
.content-card {
  width: 100%;
  height: 100%;
}
.content-card1 {
  width: calc(100% - 40px);
  height: calc(100% - 50px);
  padding: 10px 20px;
}
.chart-title {
  font-size: 15px;
  font-weight: bold;
}
.title-left {
  display: flex;
  flex-direction: column;
  align-items: flex-start;
}
.title-left > :first-child {
  font-size: 16px;
  font-weight: bold;
}
.title-right {
  display: flex;
  align-items: center;
  border: 1px solid #9797ff;
  margin-right: 10px;
}
.title-right1 {
  display: flex;
  align-items: center;
  border: 1px solid #9797ff;
  margin-right: 20px;
}
.title-right-span {
  display: flex;
  flex-direction: column;
  align-items: flex-start;
  padding-left: 10px;
  // color: #9797ff;
}
.title-right-div {
  font-size: 20px;
  padding-left: 20px;
  font-weight: bold;
  padding-right: 10px;
  color: #23bbff;
}
</style>
